@@ -29,6 +29,7 @@ from time import time
 import requests
 from requests.auth import HTTPBasicAuth
 import warnings
+from ansible.errors import AnsibleError
 
 try:
     import json
@@ -137,7 +138,7 @@ class CloudFormsInventory(object):
             warnings.warn("No username specified, you need to specify a CloudForms username.")
 
         if config.has_option('cloudforms', 'password'):
-            self.cloudforms_pw = config.get('cloudforms', 'password')
+            self.cloudforms_pw = config.get('cloudforms', 'password', raw=True)
         else:
             self.cloudforms_pw = None
 
@@ -174,6 +175,18 @@ class CloudFormsInventory(object):
         else:
             self.cloudforms_nest_tags = False
 
+        if config.has_option('cloudforms', 'suffix'):
+            self.cloudforms_suffix = config.get('cloudforms', 'suffix')
+            if self.cloudforms_suffix[0] != '.':
+                raise AnsibleError('Leading fullstop is required for Cloudforms suffix')
+        else:
+            self.cloudforms_suffix = None
+
+        if config.has_option('cloudforms', 'prefer_ipv4'):
+            self.cloudforms_prefer_ipv4 = config.getboolean('cloudforms', 'prefer_ipv4')
+        else:
+            self.cloudforms_prefer_ipv4 = False
+
         # Ansible related
         try:
             group_patterns = config.get('ansible', 'group_patterns')
@@ -280,6 +293,9 @@ class CloudFormsInventory(object):
             print("Updating cache...")
 
         for host in self._get_hosts():
+            if self.cloudforms_suffix is not None and not host['name'].endswith(self.cloudforms_suffix):
+                host['name'] = host['name'] + self.cloudforms_suffix
+
             # Ignore VMs that are not powered on
             if host['power_state'] != 'on':
                 if self.args.debug:
@@ -351,7 +367,15 @@ class CloudFormsInventory(object):
 
             # Set ansible_ssh_host to the first available ip address
             if 'ipaddresses' in host and host['ipaddresses'] and isinstance(host['ipaddresses'], list):
-                host['ansible_ssh_host'] = host['ipaddresses'][0]
+                # If no preference for IPv4, just use the first entry
+                if not self.cloudforms_prefer_ipv4:
+                    host['ansible_ssh_host'] = host['ipaddresses'][0]
+                else:
+                    # Before we search for an IPv4 address, set using the first entry in case we don't find any
+                    host['ansible_ssh_host'] = host['ipaddresses'][0]
+                    for currenthost in host['ipaddresses']:
+                        if '.' in currenthost:
+                            host['ansible_ssh_host'] = currenthost
 
             # Create additional groups
             for key in ('location', 'type', 'vendor'):
@@ -444,7 +468,7 @@ class CloudFormsInventory(object):
         Converts 'bad' characters in a string to underscores so they can be used as Ansible groups
         """
         if self.cloudforms_clean_group_keys:
-            regex = "[^A-Za-z0-9\_]"
+            regex = r"[^A-Za-z0-9\_]"
             return re.sub(regex, "_", word.replace(" ", ""))
         else:
             return word
