@@ -67,12 +67,11 @@ based on the data obtained from CloudStack API:
   }
 
 
-usage: cloudstack.py [--list] [--host HOST] [--project PROJECT]
+usage: cloudstack.py [--list] [--host HOST] [--project PROJECT]  [--domain DOMAIN]
 """
 
 from __future__ import print_function
 
-import os
 import sys
 import argparse
 
@@ -96,33 +95,50 @@ class CloudStackInventory(object):
         parser = argparse.ArgumentParser()
         parser.add_argument('--host')
         parser.add_argument('--list', action='store_true')
+        parser.add_argument('--tag', help="Filter machines by a tag. Should be in the form key=value.")
         parser.add_argument('--project')
+        parser.add_argument('--domain')
 
         options = parser.parse_args()
         try:
             self.cs = CloudStack(**read_config())
-        except CloudStackException as e:
+        except CloudStackException:
             print("Error: Could not connect to CloudStack API", file=sys.stderr)
 
-        project_id = ''
+        domain_id = None
+        if options.domain:
+            domain_id = self.get_domain_id(options.domain)
+
+        project_id = None
         if options.project:
-            project_id = self.get_project_id(options.project)
+            project_id = self.get_project_id(options.project, domain_id)
 
         if options.host:
-            data = self.get_host(options.host, project_id)
+            data = self.get_host(options.host, project_id, domain_id)
             print(json.dumps(data, indent=2))
 
         elif options.list:
-            data = self.get_list(project_id)
+            tags = dict()
+            if options.tag:
+                tags['tags[0].key'], tags['tags[0].value'] = options.tag.split('=')
+            data = self.get_list(project_id, domain_id, **tags)
             print(json.dumps(data, indent=2))
         else:
-            print("usage: --list | --host <hostname> [--project <project>]",
+            print("usage: --list [--tag <tag>] | --host <hostname> [--project <project>] [--domain <domain_path>]",
                   file=sys.stderr)
             sys.exit(1)
 
+    def get_domain_id(self, domain):
+        domains = self.cs.listDomains(listall=True)
+        if domains:
+            for d in domains['domain']:
+                if d['path'].lower() == domain.lower():
+                    return d['id']
+        print("Error: Domain %s not found." % domain, file=sys.stderr)
+        sys.exit(1)
 
-    def get_project_id(self, project):
-        projects = self.cs.listProjects()
+    def get_project_id(self, project, domain_id=None):
+        projects = self.cs.listProjects(domainid=domain_id)
         if projects:
             for p in projects['project']:
                 if p['name'] == project or p['id'] == project:
@@ -130,9 +146,8 @@ class CloudStackInventory(object):
         print("Error: Project %s not found." % project, file=sys.stderr)
         sys.exit(1)
 
-
-    def get_host(self, name, project_id=''):
-        hosts = self.cs.listVirtualMachines(projectid=project_id)
+    def get_host(self, name, project_id=None, domain_id=None, **kwargs):
+        hosts = self.cs.listVirtualMachines(projectid=project_id, domainid=domain_id, **kwargs)
         data = {}
         if not hosts:
             return data
@@ -165,37 +180,45 @@ class CloudStackInventory(object):
                     })
                     if nic['isdefault']:
                         data['default_ip'] = nic['ipaddress']
-                break;
+                break
         return data
 
-
-    def get_list(self, project_id=''):
+    def get_list(self, project_id=None, domain_id=None, **kwargs):
         data = {
             'all': {
                 'hosts': [],
-                },
+            },
             '_meta': {
                 'hostvars': {},
-                },
-            }
+            },
+        }
 
-        groups = self.cs.listInstanceGroups(projectid=project_id)
+        groups = self.cs.listInstanceGroups(projectid=project_id, domainid=domain_id)
         if groups:
             for group in groups['instancegroup']:
                 group_name = group['name']
-                if group_name and not group_name in data:
+                if group_name and group_name not in data:
                     data[group_name] = {
-                            'hosts': []
-                        }
+                        'hosts': []
+                    }
 
-        hosts = self.cs.listVirtualMachines(projectid=project_id)
+        hosts = self.cs.listVirtualMachines(projectid=project_id, domainid=domain_id, **kwargs)
         if not hosts:
             return data
         for host in hosts['virtualmachine']:
             host_name = host['displayname']
             data['all']['hosts'].append(host_name)
             data['_meta']['hostvars'][host_name] = {}
+
+            # Make a group per zone
             data['_meta']['hostvars'][host_name]['zone'] = host['zonename']
+            group_name = host['zonename']
+            if group_name not in data:
+                data[group_name] = {
+                    'hosts': []
+                }
+            data[group_name]['hosts'].append(host_name)
+
             if 'group' in host:
                 data['_meta']['hostvars'][host_name]['group'] = host['group']
             data['_meta']['hostvars'][host_name]['state'] = host['state']
@@ -219,7 +242,7 @@ class CloudStackInventory(object):
                     'netmask': nic['netmask'],
                     'gateway': nic['gateway'],
                     'type': nic['type'],
-                    })
+                })
                 if nic['isdefault']:
                     data['_meta']['hostvars'][host_name]['default_ip'] = nic['ipaddress']
 
