@@ -23,23 +23,24 @@ from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
 import os.path
+import re
+import shutil
 import sys
-import yaml
 import time
+import yaml
 
-from collections import defaultdict
-from jinja2 import Environment
+from jinja2 import Environment, FileSystemLoader
 
 import ansible.constants as C
 from ansible.cli import CLI
 from ansible.errors import AnsibleError, AnsibleOptionsError
 from ansible.galaxy import Galaxy
 from ansible.galaxy.api import GalaxyAPI
-from ansible.galaxy.role import GalaxyRole
 from ansible.galaxy.login import GalaxyLogin
+from ansible.galaxy.role import GalaxyRole
 from ansible.galaxy.token import GalaxyToken
+from ansible.module_utils._text import to_text
 from ansible.playbook.role.requirement import RoleRequirement
-from ansible.utils.unicode import to_unicode
 
 try:
     from __main__ import display
@@ -47,9 +48,11 @@ except ImportError:
     from ansible.utils.display import Display
     display = Display()
 
-class GalaxyCLI(CLI):
 
-    SKIP_INFO_KEYS = ("name", "description", "readme_html", "related", "summary_fields", "average_aw_composite", "average_aw_score", "url" )
+class GalaxyCLI(CLI):
+    '''command to manage Ansible roles in shared repostories, the default of which is Ansible Galaxy *https://galaxy.ansible.com*.'''
+
+    SKIP_INFO_KEYS = ("name", "description", "readme_html", "related", "summary_fields", "average_aw_composite", "average_aw_score", "url")
     VALID_ACTIONS = ("delete", "import", "info", "init", "install", "list", "login", "remove", "search", "setup")
 
     def __init__(self, args):
@@ -57,20 +60,9 @@ class GalaxyCLI(CLI):
         self.galaxy = None
         super(GalaxyCLI, self).__init__(args)
 
-    def parse(self):
-        ''' create an options parser for bin/ansible '''
+    def set_action(self):
 
-        self.parser = CLI.base_parser(
-            usage = "usage: %%prog [%s] [--help] [options] ..." % "|".join(self.VALID_ACTIONS),
-            epilog = "\nSee '%s <command> --help' for more information on a specific command.\n\n" % os.path.basename(sys.argv[0])
-        )
-
-
-        self.set_action()
-
-        # common
-        self.parser.add_option('-s', '--server', dest='api_server', default=C.GALAXY_SERVER, help='The API server destination')
-        self.parser.add_option('-c', '--ignore-certs', action='store_true', dest='ignore_certs', default=C.GALAXY_IGNORE_CERTS, help='Ignore SSL certificate validation errors.')
+        super(GalaxyCLI, self).set_action()
 
         # specific to actions
         if self.action == "delete":
@@ -78,16 +70,25 @@ class GalaxyCLI(CLI):
         elif self.action == "import":
             self.parser.set_usage("usage: %prog import [options] github_user github_repo")
             self.parser.add_option('--no-wait', dest='wait', action='store_false', default=True, help='Don\'t wait for import results.')
-            self.parser.add_option('--branch', dest='reference', help='The name of a branch to import. Defaults to the repository\'s default branch (usually master)')
-            self.parser.add_option('--status', dest='check_status', action='store_true', default=False, help='Check the status of the most recent import request for given github_user/github_repo.')
+            self.parser.add_option('--branch', dest='reference',
+                                   help='The name of a branch to import. Defaults to the repository\'s default branch (usually master)')
+            self.parser.add_option('--role-name', dest='role_name', help='The name the role should have, if different than the repo name')
+            self.parser.add_option('--status', dest='check_status', action='store_true', default=False,
+                                   help='Check the status of the most recent import request for given github_user/github_repo.')
         elif self.action == "info":
             self.parser.set_usage("usage: %prog info [options] role_name[,version]")
         elif self.action == "init":
             self.parser.set_usage("usage: %prog init [options] role_name")
-            self.parser.add_option('-p', '--init-path', dest='init_path', default="./", help='The path in which the skeleton role will be created. The default is the current working directory.')
+            self.parser.add_option('--init-path', dest='init_path', default="./",
+                                   help='The path in which the skeleton role will be created. The default is the current working directory.')
+            self.parser.add_option('--container-enabled', dest='container_enabled', action='store_true', default=False,
+                                   help='Initialize the skeleton role with default contents for a Container Enabled role.')
+            self.parser.add_option('--role-skeleton', dest='role_skeleton', default=C.GALAXY_ROLE_SKELETON,
+                                   help='The path to a role skeleton that the new role should be based upon.')
         elif self.action == "install":
             self.parser.set_usage("usage: %prog install [options] [-r FILE | role_name(s)[,version] | scm+role_repo_url[,version] | tar_file(s)]")
-            self.parser.add_option('-i', '--ignore-errors', dest='ignore_errors', action='store_true', default=False, help='Ignore errors and continue with the next specified role.')
+            self.parser.add_option('-i', '--ignore-errors', dest='ignore_errors', action='store_true', default=False,
+                                   help='Ignore errors and continue with the next specified role.')
             self.parser.add_option('-n', '--no-deps', dest='no_deps', action='store_true', default=False, help='Don\'t download roles listed as dependencies')
             self.parser.add_option('-r', '--role-file', dest='role_file', help='A file containing a list of roles to be imported')
         elif self.action == "remove":
@@ -98,32 +99,48 @@ class GalaxyCLI(CLI):
             self.parser.set_usage("usage: %prog login [options]")
             self.parser.add_option('--github-token', dest='token', default=None, help='Identify with github token rather than username and password.')
         elif self.action == "search":
-            self.parser.set_usage("usage: %prog search [searchterm1 searchterm2] [--galaxy-tags galaxy_tag1,galaxy_tag2] [--platforms platform1,platform2] [--author username]")
+            self.parser.set_usage("usage: %prog search [searchterm1 searchterm2] [--galaxy-tags galaxy_tag1,galaxy_tag2] [--platforms platform1,platform2] "
+                                  "[--author username]")
             self.parser.add_option('--platforms', dest='platforms', help='list of OS platforms to filter by')
-            self.parser.add_option('--galaxy-tags', dest='tags', help='list of galaxy tags to filter by')
+            self.parser.add_option('--galaxy-tags', dest='galaxy_tags', help='list of galaxy tags to filter by')
             self.parser.add_option('--author', dest='author', help='GitHub username')
         elif self.action == "setup":
             self.parser.set_usage("usage: %prog setup [options] source github_user github_repo secret")
-            self.parser.add_option('--remove', dest='remove_id', default=None, help='Remove the integration matching the provided ID value. Use --list to see ID values.')
+            self.parser.add_option('--remove', dest='remove_id', default=None,
+                                   help='Remove the integration matching the provided ID value. Use --list to see ID values.')
             self.parser.add_option('--list', dest="setup_list", action='store_true', default=False, help='List all of your integrations.')
 
         # options that apply to more than one action
         if self.action in ['init', 'info']:
-            self.parser.add_option( '--offline', dest='offline', default=False, action='store_true', help="Don't query the galaxy API when creating roles")
+            self.parser.add_option('--offline', dest='offline', default=False, action='store_true', help="Don't query the galaxy API when creating roles")
 
-        if not self.action in ("delete","import","init","login","setup"):
+        if self.action not in ("delete", "import", "init", "login", "setup"):
             # NOTE: while the option type=str, the default is a list, and the
             # callback will set the value to a list.
-            self.parser.add_option('-p', '--roles-path', dest='roles_path', action="callback", callback=CLI.expand_paths, type=str, default=C.DEFAULT_ROLES_PATH,
-                help='The path to the directory containing your roles. The default is the roles_path configured in your ansible.cfg file (/etc/ansible/roles if not configured)')
-
-        if self.action in ("init","install"):
+            self.parser.add_option('-p', '--roles-path', dest='roles_path', action="callback", callback=CLI.unfrack_paths, default=C.DEFAULT_ROLES_PATH,
+                                   help='The path to the directory containing your roles. The default is the roles_path configured in your ansible.cfg'
+                                        'file (/etc/ansible/roles if not configured)', type='str')
+        if self.action in ("init", "install"):
             self.parser.add_option('-f', '--force', dest='force', action='store_true', default=False, help='Force overwriting an existing role')
 
-        self.options, self.args =self.parser.parse_args()
+    def parse(self):
+        ''' create an options parser for bin/ansible '''
+
+        self.parser = CLI.base_parser(
+            usage="usage: %%prog [%s] [--help] [options] ..." % "|".join(self.VALID_ACTIONS),
+            epilog="\nSee '%s <command> --help' for more information on a specific command.\n\n" % os.path.basename(sys.argv[0])
+        )
+
+        # common
+        self.parser.add_option('-s', '--server', dest='api_server', default=C.GALAXY_SERVER, help='The API server destination')
+        self.parser.add_option('-c', '--ignore-certs', action='store_true', dest='ignore_certs', default=C.GALAXY_IGNORE_CERTS,
+                               help='Ignore SSL certificate validation errors.')
+        self.set_action()
+
+        super(GalaxyCLI, self).parse()
+
         display.verbosity = self.options.verbosity
         self.galaxy = Galaxy(self.options)
-        return True
 
     def run(self):
 
@@ -137,12 +154,12 @@ class GalaxyCLI(CLI):
         Exits with the specified return code unless the
         option --ignore-errors was specified
         """
-        if not self.get_opt("ignore_errors", False):
+        if not self.options.ignore_errors:
             raise AnsibleError('- you can use --ignore-errors to skip failed roles and finish processing the list.')
 
     def _display_role_info(self, role_info):
 
-        text = [u"", u"Role: %s" % to_unicode(role_info['name'])]
+        text = [u"", u"Role: %s" % to_text(role_info['name'])]
         text.append(u"\tdescription: %s" % role_info.get('description', ''))
 
         for k in sorted(role_info.keys()):
@@ -151,7 +168,6 @@ class GalaxyCLI(CLI):
                 continue
 
             if isinstance(role_info[k], dict):
-                text += "\t%s: \n" % (k)
                 text.append(u"\t%s:" % (k))
                 for key in sorted(role_info[k].keys()):
                     if key in self.SKIP_INFO_KEYS:
@@ -168,13 +184,12 @@ class GalaxyCLI(CLI):
 
     def execute_init(self):
         """
-        Executes the init action, which creates the skeleton framework
-        of a role that complies with the galaxy metadata format.
+        creates the skeleton framework of a role that complies with the galaxy metadata format.
         """
 
-        init_path  = self.get_opt('init_path', './')
-        force      = self.get_opt('force', False)
-        offline    = self.get_opt('offline', False)
+        init_path = self.options.init_path
+        force = self.options.force
+        role_skeleton = self.options.role_skeleton
 
         role_name = self.args.pop(0).strip() if self.args else None
         if not role_name:
@@ -185,97 +200,70 @@ class GalaxyCLI(CLI):
                 raise AnsibleError("- the path %s already exists, but is a file - aborting" % role_path)
             elif not force:
                 raise AnsibleError("- the directory %s already exists."
-                            "you can use --force to re-initialize this directory,\n"
-                            "however it will reset any main.yml files that may have\n"
-                            "been modified there already." % role_path)
+                                   "you can use --force to re-initialize this directory,\n"
+                                   "however it will reset any main.yml files that may have\n"
+                                   "been modified there already." % role_path)
 
-        # create default README.md
+        inject_data = dict(
+            role_name=role_name,
+            author='your name',
+            description='your description',
+            company='your company (optional)',
+            license='license (GPLv2, CC-BY, etc)',
+            issue_tracker_url='http://example.com/issue/tracker',
+            min_ansible_version='1.2',
+            container_enabled=self.options.container_enabled
+        )
+
+        # create role directory
         if not os.path.exists(role_path):
             os.makedirs(role_path)
-        readme_path = os.path.join(role_path, "README.md")
-        f = open(readme_path, "wb")
-        f.write(self.galaxy.default_readme)
-        f.close()
 
-        # create default .travis.yml
-        travis = Environment().from_string(self.galaxy.default_travis).render()
-        f = open(os.path.join(role_path, '.travis.yml'), 'w')
-        f.write(travis)
-        f.close()
+        if role_skeleton is not None:
+            skeleton_ignore_expressions = C.GALAXY_ROLE_SKELETON_IGNORE
+        else:
+            role_skeleton = self.galaxy.default_role_skeleton_path
+            skeleton_ignore_expressions = ['^.*/.git_keep$']
 
-        for dir in GalaxyRole.ROLE_DIRS:
-            dir_path = os.path.join(init_path, role_name, dir)
-            main_yml_path = os.path.join(dir_path, 'main.yml')
+        role_skeleton = os.path.expanduser(role_skeleton)
+        skeleton_ignore_re = [re.compile(x) for x in skeleton_ignore_expressions]
 
-            # create the directory if it doesn't exist already
-            if not os.path.exists(dir_path):
-                os.makedirs(dir_path)
+        template_env = Environment(loader=FileSystemLoader(role_skeleton))
 
-            # now create the main.yml file for that directory
-            if dir == "meta":
-                # create a skeleton meta/main.yml with a valid galaxy_info
-                # datastructure in place, plus with all of the available
-                # platforms included (but commented out), the galaxy_tags
-                # list, and the dependencies section
-                platforms = []
-                if not offline:
-                    platforms = self.api.get_list("platforms") or []
+        for root, dirs, files in os.walk(role_skeleton, topdown=True):
+            rel_root = os.path.relpath(root, role_skeleton)
+            in_templates_dir = rel_root.split(os.sep, 1)[0] == 'templates'
+            dirs[:] = [d for d in dirs if not any(r.match(d) for r in skeleton_ignore_re)]
 
-                # group the list of platforms from the api based
-                # on their names, with the release field being
-                # appended to a list of versions
-                platform_groups = defaultdict(list)
-                for platform in platforms:
-                    platform_groups[platform['name']].append(platform['release'])
-                    platform_groups[platform['name']].sort()
+            for f in files:
+                filename, ext = os.path.splitext(f)
+                if any(r.match(os.path.join(rel_root, f)) for r in skeleton_ignore_re):
+                    continue
+                elif ext == ".j2" and not in_templates_dir:
+                    src_template = os.path.join(rel_root, f)
+                    dest_file = os.path.join(role_path, rel_root, filename)
+                    template_env.get_template(src_template).stream(inject_data).dump(dest_file)
+                else:
+                    f_rel_path = os.path.relpath(os.path.join(root, f), role_skeleton)
+                    shutil.copyfile(os.path.join(root, f), os.path.join(role_path, f_rel_path))
 
-                inject = dict(
-                    author = 'your name',
-                    description = 'your description',
-                    company = 'your company (optional)',
-                    license = 'license (GPLv2, CC-BY, etc)',
-                    issue_tracker_url = 'http://example.com/issue/tracker',
-                    min_ansible_version = '1.2',
-                    platforms = platform_groups,
-                )
-                rendered_meta = Environment().from_string(self.galaxy.default_meta).render(inject)
-                f = open(main_yml_path, 'w')
-                f.write(rendered_meta)
-                f.close()
-                pass
-            elif dir == "tests":
-                # create tests/test.yml
-                inject = dict(
-                    role_name = role_name
-                )
-                playbook = Environment().from_string(self.galaxy.default_test).render(inject)
-                f = open(os.path.join(dir_path, 'test.yml'), 'w')
-                f.write(playbook)
-                f.close()
+            for d in dirs:
+                dir_path = os.path.join(role_path, rel_root, d)
+                if not os.path.exists(dir_path):
+                    os.makedirs(dir_path)
 
-                # create tests/inventory
-                f = open(os.path.join(dir_path, 'inventory'), 'w')
-                f.write('localhost')
-                f.close()
-            elif dir not in ('files','templates'):
-                # just write a (mostly) empty YAML file for main.yml
-                f = open(main_yml_path, 'w')
-                f.write('---\n# %s file for %s\n' % (dir,role_name))
-                f.close()
         display.display("- %s was created successfully" % role_name)
 
     def execute_info(self):
         """
-        Executes the info action. This action prints out detailed
-        information about an installed role as well as info available
-        from the galaxy API.
+        prints out detailed information about an installed role as well as info available from the galaxy API.
         """
 
         if len(self.args) == 0:
             # the user needs to specify a role
             raise AnsibleOptionsError("- you must specify a user/role name")
 
-        roles_path = self.get_opt("roles_path")
+        roles_path = self.options.roles_path
 
         data = ''
         for role in self.args:
@@ -301,12 +289,12 @@ class GalaxyCLI(CLI):
                 role_info.update(gr.metadata)
 
             req = RoleRequirement()
-            role_spec= req.role_yaml_parse({'role': role})
+            role_spec = req.role_yaml_parse({'role': role})
             if role_spec:
                 role_info.update(role_spec)
 
             data = self._display_role_info(role_info)
-            ### FIXME: This is broken in both 1.9 and 2.0 as
+            # FIXME: This is broken in both 1.9 and 2.0 as
             # _display_role_info() always returns something
             if not data:
                 data = u"\n- the role %s was not found" % role
@@ -315,25 +303,17 @@ class GalaxyCLI(CLI):
 
     def execute_install(self):
         """
-        Executes the installation action. The args list contains the
-        roles to be installed, unless -f was specified. The list of roles
-        can be a name (which will be downloaded via the galaxy API and github),
-        or it can be a local .tar.gz file.
+        uses the args list of roles to be installed, unless -f was specified. The list of roles
+        can be a name (which will be downloaded via the galaxy API and github), or it can be a local .tar.gz file.
         """
-
-        role_file  = self.get_opt("role_file", None)
+        role_file = self.options.role_file
 
         if len(self.args) == 0 and role_file is None:
-            # the user needs to specify one of either --role-file
-            # or specify a single user/role name
+            # the user needs to specify one of either --role-file or specify a single user/role name
             raise AnsibleOptionsError("- you must specify a user/role name or a roles file")
-        elif len(self.args) == 1 and role_file is not None:
-            # using a role file is mutually exclusive of specifying
-            # the role name on the command line
-            raise AnsibleOptionsError("- please specify a user/role name, or a roles file, but not both")
 
-        no_deps    = self.get_opt("no_deps", False)
-        force      = self.get_opt('force', False)
+        no_deps = self.options.no_deps
+        force = self.options.force
 
         roles_left = []
         if role_file:
@@ -341,7 +321,7 @@ class GalaxyCLI(CLI):
                 f = open(role_file, 'r')
                 if role_file.endswith('.yaml') or role_file.endswith('.yml'):
                     try:
-                        required_roles =  yaml.safe_load(f.read())
+                        required_roles = yaml.safe_load(f.read())
                     except Exception as e:
                         raise AnsibleError("Unable to load data from the requirements file: %s" % role_file)
 
@@ -349,13 +329,24 @@ class GalaxyCLI(CLI):
                         raise AnsibleError("No roles found in file: %s" % role_file)
 
                     for role in required_roles:
-                        role = RoleRequirement.role_yaml_parse(role)
-                        display.vvv('found role %s in yaml file' % str(role))
-                        if 'name' not in role and 'scm' not in role:
-                            raise AnsibleError("Must specify name or src for role")
-                        roles_left.append(GalaxyRole(self.galaxy, **role))
+                        if "include" not in role:
+                            role = RoleRequirement.role_yaml_parse(role)
+                            display.vvv("found role %s in yaml file" % str(role))
+                            if "name" not in role and "scm" not in role:
+                                raise AnsibleError("Must specify name or src for role")
+                            roles_left.append(GalaxyRole(self.galaxy, **role))
+                        else:
+                            with open(role["include"]) as f_include:
+                                try:
+                                    roles_left += [
+                                        GalaxyRole(self.galaxy, **r) for r in
+                                        (RoleRequirement.role_yaml_parse(i) for i in yaml.safe_load(f_include))
+                                    ]
+                                except Exception as e:
+                                    msg = "Unable to load data from the include requirements file: %s %s"
+                                    raise AnsibleError(msg % (role_file, e))
                 else:
-                    display.deprecated("going forward only the yaml format will be supported")
+                    display.deprecated("going forward only the yaml format will be supported", version="2.6")
                     # roles listed in a file, one per line
                     for rline in f.readlines():
                         if rline.startswith("#") or rline.strip() == '':
@@ -365,7 +356,7 @@ class GalaxyCLI(CLI):
                         roles_left.append(GalaxyRole(self.galaxy, **role))
                 f.close()
             except (IOError, OSError) as e:
-                display.error('Unable to open %s: %s' % (role_file, str(e)))
+                raise AnsibleError('Unable to open %s: %s' % (role_file, str(e)))
         else:
             # roles were specified directly, so we'll just go out grab them
             # (and their dependencies, unless the user doesn't want us to).
@@ -374,12 +365,29 @@ class GalaxyCLI(CLI):
                 roles_left.append(GalaxyRole(self.galaxy, **role))
 
         for role in roles_left:
-            display.vvv('Installing role %s ' % role.name)
+            # only process roles in roles files when names matches if given
+            if role_file and self.args and role.name not in self.args:
+                display.vvv('Skipping role %s' % role.name)
+                continue
+
+            display.vvv('Processing role %s ' % role.name)
+
             # query the galaxy API for the role data
 
-            if role.install_info is not None and not force:
-                display.display('- %s is already installed, skipping.' % role.name)
-                continue
+            if role.install_info is not None:
+                if role.install_info['version'] != role.version or force:
+                    if force:
+                        display.display('- changing role %s from %s to %s' %
+                                        (role.name, role.install_info['version'], role.version or "unspecified"))
+                        role.remove()
+                    else:
+                        display.warning('- %s (%s) is already installed - use --force to change version to %s' %
+                                        (role.name, role.install_info['version'], role.version or "unspecified"))
+                        continue
+                else:
+                    if not force:
+                        display.display('- %s is already installed, skipping.' % str(role))
+                        continue
 
             try:
                 installed = role.install()
@@ -390,24 +398,31 @@ class GalaxyCLI(CLI):
 
             # install dependencies, if we want them
             if not no_deps and installed:
-                role_dependencies = role.metadata.get('dependencies') or []
-                for dep in role_dependencies:
-                    display.debug('Installing dep %s' % dep)
-                    dep_req = RoleRequirement()
-                    dep_info = dep_req.role_yaml_parse(dep)
-                    dep_role = GalaxyRole(self.galaxy, **dep_info)
-                    if '.' not in dep_role.name and '.' not in dep_role.src and dep_role.scm is None:
-                        # we know we can skip this, as it's not going to
-                        # be found on galaxy.ansible.com
-                        continue
-                    if dep_role.install_info is None or force:
-                        if dep_role not in roles_left:
-                            display.display('- adding dependency: %s' % dep_role.name)
-                            roles_left.append(dep_role)
+                if not role.metadata:
+                    display.warning("Meta file %s is empty. Skipping dependencies." % role.path)
+                else:
+                    role_dependencies = role.metadata.get('dependencies') or []
+                    for dep in role_dependencies:
+                        display.debug('Installing dep %s' % dep)
+                        dep_req = RoleRequirement()
+                        dep_info = dep_req.role_yaml_parse(dep)
+                        dep_role = GalaxyRole(self.galaxy, **dep_info)
+                        if '.' not in dep_role.name and '.' not in dep_role.src and dep_role.scm is None:
+                            # we know we can skip this, as it's not going to
+                            # be found on galaxy.ansible.com
+                            continue
+                        if dep_role.install_info is None:
+                            if dep_role not in roles_left:
+                                display.display('- adding dependency: %s' % str(dep_role))
+                                roles_left.append(dep_role)
+                            else:
+                                display.display('- dependency %s already pending installation.' % dep_role.name)
                         else:
-                            display.display('- dependency %s already pending installation.' % dep_role.name)
-                    else:
-                        display.display('- dependency %s is already installed, skipping.' % dep_role.name)
+                            if dep_role.install_info['version'] != dep_role.version:
+                                display.warning('- dependency %s from role %s differs from already installed version (%s), skipping' %
+                                                (str(dep_role), role.name, dep_role.install_info['version']))
+                            else:
+                                display.display('- dependency %s is already installed, skipping.' % dep_role.name)
 
             if not installed:
                 display.warning("- %s was NOT installed successfully." % role.name)
@@ -417,8 +432,7 @@ class GalaxyCLI(CLI):
 
     def execute_remove(self):
         """
-        Executes the remove action. The args list contains the list
-        of roles to be removed. This list can contain more than one role.
+        removes the list of roles passed as arguments from the local system.
         """
 
         if len(self.args) == 0:
@@ -438,10 +452,7 @@ class GalaxyCLI(CLI):
 
     def execute_list(self):
         """
-        Executes the list action. The args list can contain zero
-        or one role. If one is specified, only that role will be
-        shown, otherwise all roles in the specified directory will
-        be shown.
+        lists the roles installed on the local system or matches a single role passed as an argument.
         """
 
         if len(self.args) > 1:
@@ -464,7 +475,7 @@ class GalaxyCLI(CLI):
                 display.display("- the role %s was not found" % name)
         else:
             # show all valid roles in the roles_path directory
-            roles_path = self.get_opt('roles_path')
+            roles_path = self.options.roles_path
             for path in roles_path:
                 role_path = os.path.expanduser(path)
                 if not os.path.exists(role_path):
@@ -485,20 +496,21 @@ class GalaxyCLI(CLI):
         return 0
 
     def execute_search(self):
+        ''' searches for roles on the Ansible Galaxy server'''
         page_size = 1000
         search = None
 
         if len(self.args):
             terms = []
             for i in range(len(self.args)):
-               terms.append(self.args.pop())
+                terms.append(self.args.pop())
             search = '+'.join(terms[::-1])
 
-        if not search and not self.options.platforms and not self.options.tags and not self.options.author:
+        if not search and not self.options.platforms and not self.options.galaxy_tags and not self.options.author:
             raise AnsibleError("Invalid query. At least one search term, platform, galaxy tag or author must be provided.")
 
         response = self.api.search_roles(search, platforms=self.options.platforms,
-            tags=self.options.tags, author=self.options.author, page_size=page_size)
+                                         tags=self.options.galaxy_tags, author=self.options.author, page_size=page_size)
 
         if response['count'] == 0:
             display.display("No roles match your search.", color=C.COLOR_ERROR)
@@ -529,18 +541,21 @@ class GalaxyCLI(CLI):
 
     def execute_login(self):
         """
-        Verify user's identify via Github and retreive an auth token from Galaxy.
+        verify user's identify via Github and retrieve an auth token from Ansible Galaxy.
         """
         # Authenticate with github and retrieve a token
         if self.options.token is None:
-            login = GalaxyLogin(self.galaxy)
-            github_token = login.create_github_token()
+            if C.GALAXY_TOKEN:
+                github_token = C.GALAXY_TOKEN
+            else:
+                login = GalaxyLogin(self.galaxy)
+                github_token = login.create_github_token()
         else:
             github_token = self.options.token
 
         galaxy_response = self.api.authenticate(github_token)
 
-        if self.options.token is None:
+        if self.options.token is None and C.GALAXY_TOKEN is None:
             # Remove the token we created
             login.remove_github_token()
 
@@ -548,18 +563,16 @@ class GalaxyCLI(CLI):
         token = GalaxyToken()
         token.set(galaxy_response['token'])
 
-        display.display("Succesfully logged into Galaxy as %s" % galaxy_response['username'])
+        display.display("Successfully logged into Galaxy as %s" % galaxy_response['username'])
         return 0
 
     def execute_import(self):
-        """
-        Import a role into Galaxy
-        """
+        """ used to import a role into Ansible Galaxy """
 
         colors = {
-            'INFO':    'normal',
+            'INFO': 'normal',
             'WARNING': C.COLOR_WARN,
-            'ERROR':   C.COLOR_ERROR,
+            'ERROR': C.COLOR_ERROR,
             'SUCCESS': C.COLOR_OK,
             'FAILED': C.COLOR_ERROR,
         }
@@ -567,29 +580,30 @@ class GalaxyCLI(CLI):
         if len(self.args) < 2:
             raise AnsibleError("Expected a github_username and github_repository. Use --help.")
 
-        github_repo = self.args.pop()
-        github_user = self.args.pop()
+        github_repo = to_text(self.args.pop(), errors='surrogate_or_strict')
+        github_user = to_text(self.args.pop(), errors='surrogate_or_strict')
 
         if self.options.check_status:
             task = self.api.get_import_task(github_user=github_user, github_repo=github_repo)
         else:
             # Submit an import request
-            task = self.api.create_import_task(github_user, github_repo, reference=self.options.reference)
+            task = self.api.create_import_task(github_user, github_repo, reference=self.options.reference, role_name=self.options.role_name)
 
             if len(task) > 1:
                 # found multiple roles associated with github_user/github_repo
-                display.display("WARNING: More than one Galaxy role associated with Github repo %s/%s." % (github_user,github_repo),
-                    color='yellow')
+                display.display("WARNING: More than one Galaxy role associated with Github repo %s/%s." % (github_user, github_repo),
+                                color='yellow')
                 display.display("The following Galaxy roles are being updated:" + u'\n', color=C.COLOR_CHANGED)
                 for t in task:
-                    display.display('%s.%s' % (t['summary_fields']['role']['namespace'],t['summary_fields']['role']['name']), color=C.COLOR_CHANGED)
-                display.display(u'\n' + "To properly namespace this role, remove each of the above and re-import %s/%s from scratch" % (github_user,github_repo), color=C.COLOR_CHANGED)
+                    display.display('%s.%s' % (t['summary_fields']['role']['namespace'], t['summary_fields']['role']['name']), color=C.COLOR_CHANGED)
+                display.display(u'\nTo properly namespace this role, remove each of the above and re-import %s/%s from scratch' % (github_user, github_repo),
+                                color=C.COLOR_CHANGED)
                 return 0
             # found a single role as expected
             display.display("Successfully submitted import request %d" % task[0]['id'])
             if not self.options.wait:
                 display.display("Role name: %s" % task[0]['summary_fields']['role']['name'])
-                display.display("Repo: %s/%s" % (task[0]['github_user'],task[0]['github_repo']))
+                display.display("Repo: %s/%s" % (task[0]['github_user'], task[0]['github_repo']))
 
         if self.options.check_status or self.options.wait:
             # Get the status of the import
@@ -609,9 +623,7 @@ class GalaxyCLI(CLI):
         return 0
 
     def execute_setup(self):
-        """
-        Setup an integration from Github or Travis
-        """
+        """ Setup an integration from Github or Travis for Ansible Galaxy roles"""
 
         if self.options.setup_list:
             # List existing integration secrets
@@ -624,7 +636,7 @@ class GalaxyCLI(CLI):
             display.display("---------- ---------- ----------", color=C.COLOR_OK)
             for secret in secrets:
                 display.display("%-10s %-10s %s/%s" % (secret['id'], secret['source'], secret['github_user'],
-                    secret['github_repo']),color=C.COLOR_OK)
+                                                       secret['github_repo']), color=C.COLOR_OK)
             return 0
 
         if self.options.remove_id:
@@ -635,7 +647,6 @@ class GalaxyCLI(CLI):
 
         if len(self.args) < 4:
             raise AnsibleError("Missing one or more arguments. Expecting: source github_user github_repo secret")
-            return 0
 
         secret = self.args.pop()
         github_repo = self.args.pop()
@@ -648,9 +659,7 @@ class GalaxyCLI(CLI):
         return 0
 
     def execute_delete(self):
-        """
-        Delete a role from galaxy.ansible.com
-        """
+        """ Delete a role from Ansible Galaxy. """
 
         if len(self.args) < 2:
             raise AnsibleError("Missing one or more arguments. Expected: github_user github_repo")
@@ -664,7 +673,7 @@ class GalaxyCLI(CLI):
             display.display("ID     User            Name")
             display.display("------ --------------- ----------")
             for role in resp['deleted_roles']:
-                display.display("%-8s %-15s %s" % (role.id,role.namespace,role.name))
+                display.display("%-8s %-15s %s" % (role.id, role.namespace, role.name))
 
         display.display(resp['status'])
 
