@@ -42,7 +42,7 @@ options:
     required: false
   cleanup:
     description:
-      - Use with I(detach) to remove the container after successful execution.
+      - Use with I(detach=false) to remove the container after successful execution.
     default: false
     required: false
     version_added: "2.2"
@@ -100,6 +100,12 @@ options:
       - List of custom DNS search domains.
     default: null
     required: false
+  domainname:
+    description:
+      - Container domainname.
+    default: null
+    required: false
+    version_added: "2.5"
   env:
     description:
       - Dictionary of key,value pairs.
@@ -264,6 +270,12 @@ options:
       - none
     default: null
     required: false
+  userns_mode:
+     description:
+       - User namespace to use
+     default: null
+     required: false
+     version_added: "2.5"
   networks:
      description:
        - List of networks the container belongs to.
@@ -725,6 +737,7 @@ class TaskParameters(DockerBaseClass):
         self.dns_servers = None
         self.dns_opts = None
         self.dns_search_domains = None
+        self.domainname = None
         self.env = None
         self.env_file = None
         self.entrypoint = None
@@ -751,6 +764,7 @@ class TaskParameters(DockerBaseClass):
         self.memory_swappiness = None
         self.name = None
         self.network_mode = None
+        self.userns_mode = None
         self.networks = None
         self.oom_killer = None
         self.oom_score_adj = None
@@ -853,7 +867,7 @@ class TaskParameters(DockerBaseClass):
             cpu_shares='cpu_shares',
             cpuset_cpus='cpuset_cpus',
             mem_limit='memory',
-            mem_reservation='mem_reservation',
+            mem_reservation='memory_reservation',
             memswap_limit='memory_swap',
             kernel_memory='kernel_memory',
         )
@@ -870,6 +884,7 @@ class TaskParameters(DockerBaseClass):
         '''
         create_params = dict(
             command='command',
+            domainname='domainname',
             hostname='hostname',
             user='user',
             detach='detach',
@@ -946,7 +961,7 @@ class TaskParameters(DockerBaseClass):
         Returns parameters used to create a HostConfig object
         '''
 
-        host_config_params=dict(
+        host_config_params = dict(
             port_bindings='published_ports',
             publish_all_ports='publish_all_ports',
             links='links',
@@ -956,6 +971,7 @@ class TaskParameters(DockerBaseClass):
             binds='volume_binds',
             volumes_from='volumes_from',
             network_mode='network_mode',
+            userns_mode='userns_mode',
             cap_add='capabilities',
             extra_hosts='etc_hosts',
             read_only='read_only',
@@ -1162,7 +1178,7 @@ class TaskParameters(DockerBaseClass):
 
         options = dict(
             Type=self.log_driver,
-            Config = dict()
+            Config=dict()
         )
 
         if self.log_options is not None:
@@ -1216,7 +1232,6 @@ class TaskParameters(DockerBaseClass):
         return network_id
 
 
-
 class Container(DockerBaseClass):
 
     def __init__(self, container, parameters):
@@ -1297,6 +1312,7 @@ class Container(DockerBaseClass):
         config_mapping = dict(
             auto_remove=host_config.get('AutoRemove'),
             expected_cmd=config.get('Cmd'),
+            domainname=config.get('Domainname'),
             hostname=config.get('Hostname'),
             user=config.get('User'),
             detach=detach,
@@ -1319,6 +1335,7 @@ class Container(DockerBaseClass):
             mac_address=network.get('MacAddress'),
             memory_swappiness=host_config.get('MemorySwappiness'),
             network_mode=host_config.get('NetworkMode'),
+            userns_mode=host_config.get('UsernsMode'),
             oom_killer=host_config.get('OomKillDisable'),
             oom_score_adj=host_config.get('OomScoreAdj'),
             pid_mode=host_config.get('PidMode'),
@@ -1568,7 +1585,7 @@ class Container(DockerBaseClass):
                         CgroupPermissions=parts[2],
                         PathInContainer=parts[1],
                         PathOnHost=parts[0]
-                        ))
+                    ))
         return expected_devices
 
     def _get_expected_entrypoint(self):
@@ -1967,7 +1984,14 @@ class ContainerManager(DockerBaseClass):
                     status = self.client.wait(container_id)['StatusCode']
                 else:
                     status = self.client.wait(container_id)
-                output = self.client.logs(container_id, stdout=True, stderr=True, stream=False, timestamps=False)
+                config = self.client.inspect_container(container_id)
+                logging_driver = config['HostConfig']['LogConfig']['Type']
+
+                if logging_driver == 'json-file' or logging_driver == 'journald':
+                    output = self.client.logs(container_id, stdout=True, stderr=True, stream=False, timestamps=False)
+                else:
+                    output = "Result logged using `%s` driver" % logging_driver
+
                 if status != 0:
                     self.fail(output, status=status)
                 if self.parameters.cleanup:
@@ -2055,6 +2079,7 @@ def main():
         dns_servers=dict(type='list'),
         dns_opts=dict(type='list'),
         dns_search_domains=dict(type='list'),
+        domainname=dict(type='str'),
         env=dict(type='dict'),
         env_file=dict(type='path'),
         entrypoint=dict(type='list'),
@@ -2083,6 +2108,7 @@ def main():
         memory_swappiness=dict(type='int'),
         name=dict(type='str', required=True),
         network_mode=dict(type='str'),
+        userns_mode=dict(type='str'),
         networks=dict(type='list'),
         oom_killer=dict(type='bool'),
         oom_score_adj=dict(type='int'),
