@@ -94,7 +94,7 @@ options:
         description:
             - Sets the frequency at which the software sends IGMP host query
               messages. Values can range from 1 to 18000 seconds.
-              he default is 125 seconds.
+              The default is 125 seconds.
         required: false
         default: null
     last_member_qrt:
@@ -171,11 +171,7 @@ EXAMPLES = '''
     interface: ethernet1/32
     startup_query_interval: 30
     state: present
-    username: "{{ un }}"
-    password: "{{ pwd }}"
-    host: "{{ inventory_hostname }}"
 '''
-
 RETURN = '''
 proposed:
     description: k/v pairs of parameters passed into module
@@ -236,12 +232,13 @@ changed:
     sample: true
 '''
 
-from ansible.module_utils.nxos import get_config, load_config, run_commands
-from ansible.module_utils.nxos import nxos_argument_spec, check_args
+from ansible.module_utils.network.nxos.nxos import get_config, load_config, run_commands
+from ansible.module_utils.network.nxos.nxos import nxos_argument_spec, check_args
 from ansible.module_utils.basic import AnsibleModule
 
 import re
 
+
 def execute_show_command(command, module, command_type='cli_show'):
     if command_type == 'cli_show_ascii':
         cmds = [{
@@ -358,11 +355,11 @@ def get_igmp_interface(module, interface):
     staticoif = []
     if body:
         split_body = body.split('\n')
-        route_map_regex = ('.*ip igmp static-oif route-map\s+'
-                           '(?P<route_map>\S+).*')
-        prefix_source_regex = ('.*ip igmp static-oif\s+(?P<prefix>'
-                               '((\d+.){3}\d+))(\ssource\s'
-                               '(?P<source>\S+))?.*')
+        route_map_regex = (r'.*ip igmp static-oif route-map\s+'
+                           r'(?P<route_map>\S+).*')
+        prefix_source_regex = (r'.*ip igmp static-oif\s+(?P<prefix>'
+                               r'((\d+.){3}\d+))(\ssource\s'
+                               r'(?P<source>\S+))?.*')
 
         for line in split_body:
             temp = {}
@@ -501,11 +498,11 @@ def config_remove_oif(existing, existing_oif_prefix_source):
             if each.get('prefix') and each.get('source'):
                 command = 'no ip igmp static-oif {0} source {1} '.format(
                     each.get('prefix'), each.get('source')
-                    )
+                )
             elif each.get('prefix'):
                 command = 'no ip igmp static-oif {0}'.format(
                     each.get('prefix')
-                    )
+                )
             if command:
                 commands.append(command)
             command = None
@@ -533,21 +530,17 @@ def main():
         oif_source=dict(required=False, type='str'),
         restart=dict(type='bool', default=False),
         state=dict(choices=['present', 'absent', 'default'],
-                       default='present'),
-        include_defaults=dict(default=True),
-        config=dict(),
-        save=dict(type='bool', default=False)
+                   default='present')
     )
 
     argument_spec.update(nxos_argument_spec)
 
     module = AnsibleModule(argument_spec=argument_spec,
-                                supports_check_mode=True)
+                           supports_check_mode=True)
 
     warnings = list()
     check_args(module, warnings)
 
-
     state = module.params['state']
     interface = module.params['interface']
     oif_prefix = module.params['oif_prefix']
@@ -607,7 +600,7 @@ def main():
     changed = False
     commands = []
     proposed = dict((k, v) for k, v in module.params.items()
-                     if v is not None and k in args)
+                    if v is not None and k in args)
 
     CANNOT_ABSENT = ['version', 'startup_query_interval',
                      'startup_query_count', 'robustness', 'querier_timeout',
