@@ -26,9 +26,10 @@ from ansible.playbook.base import Base
 # FIXME: loopcontrol should not inherit attributes from base, just uses it for load
 class LoopControl(Base):
 
-    _loop_var = FieldAttribute(isa='str')
+    _loop_var = FieldAttribute(isa='str', default='item')
+    _index_var = FieldAttribute(isa='str')
     _label = FieldAttribute(isa='str')
-    _pause = FieldAttribute(isa='int')
+    _pause = FieldAttribute(isa='int', default=0)
 
     def __init__(self):
         super(LoopControl, self).__init__()
