@@ -1,6 +1,22 @@
+# (c) 2017 Ansible Project
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
+DOCUMENTATION = '''
+    callback: debug
+    type: stdout
+    short_description: formated stdout/stderr display
+    description:
+      - Use this callback to sort though extensive debug output
+    version_added: "2.4"
+    extends_documentation_fragment:
+      - default_callback
+    requirements:
+      - set as stdout in configuration
+'''
+
 from ansible.plugins.callback.default import CallbackModule as CallbackModule_default
 
 
@@ -22,13 +38,13 @@ class CallbackModule(CallbackModule_default):  # pylint: disable=too-few-public-
         result['_ansible_verbose_always'] = True
 
         save = {}
-        for key in ['stdout', 'stdout_lines', 'stderr', 'stderr_lines', 'msg']:
+        for key in ['stdout', 'stdout_lines', 'stderr', 'stderr_lines', 'msg', 'module_stdout', 'module_stderr']:
             if key in result:
                 save[key] = result.pop(key)
 
         output = CallbackModule_default._dump_results(self, result)
 
-        for key in ['stdout', 'stderr', 'msg']:
+        for key in ['stdout', 'stderr', 'msg', 'module_stdout', 'module_stderr']:
             if key in save and save[key]:
                 output += '\n\n%s:\n\n%s\n' % (key.upper(), save[key])
 
