@@ -1,23 +1,34 @@
 # (c) 2016 Matt Clay <matt@mystile.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# (c) 2017 Ansible Project
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
+DOCUMENTATION = """
+    author: Matt Clay <matt@mystile.com>
+    connection: lxd
+    short_description: Run tasks in lxc containers via lxc CLI
+    description:
+        - Run commands or put/fetch files to an existing lxc container using lxc CLI
+    version_added: "2.0"
+    options:
+      remote_addr:
+        description:
+            - Container identifier
+        default: The set user as per docker's configuration
+        vars:
+            - name: ansible_host
+            - name: ansible_lxd_host
+      executable:
+        description:
+            - shell to use for execution inside container
+        default: /bin/sh
+        vars:
+            - name: ansible_executable
+            - name: ansible_lxd_executable
+"""
+
 import os
 from distutils.spawn import find_executable
 from subprocess import call, Popen, PIPE
